""" Monitored variables

    A Class for monitored variables, that is, variables that
    allow one to gather simple means and variances, either tallied or
    accumulated over time. A monitored variable is an object of the class
    and is set up using m= Monitor(). It can be given a unique name for use
    in debugging and in tracing.

    For the generator version of SimPy

    HISTORY
    2002 10 23: 1.4 added try-except to calculations of statistics
    2002 11 22: testing new now()
    2003 4  13: added Simpy to error messages
    2003 4  18: minor correction to error messages
    
"""

__version__ = '$Revision: 1.1 $ $Date: 2003/08/19 21:07:38 $ gav'

from SimPy.Simulation import now
import unittest

class Monitor:
    def __init__(self,name=''):
        self.n = 0
        self.total= 0.0
        self.ttotal = 0.0
        self.Stx = 0.0 
        self.oldx = 0.0
        self.lastTime =0.0
        self.startTime = 0.0
        self.name = name

    def tally(self,x):
        """ add x, and x^2 to the sums for the monitored variable
        """
        self.total += x
        self.ttotal += x*x
        self.n += 1

    def accum(self,x,t=None):
        """ add xdt and x^2dt to the sums for the monitored variable
        
        t is the current time
        """
        if t == None: t = now()
        self.Stx += self.oldx*(t - self.lastTime)
        self.oldx = x
        self.lastTime = t
        self.n += 1
             
    def reset(self,t=None):
        """reset the sums and counts for the monitored variable """
        if t == None: t = now()
        self.n = 0
        self.total = 0.0
        self.ttotal = 0.0
        self.Stx = 0.0 
        self.oldx = 0.0
        self.lastTime = t
        self.startTime = t

    def count(self):
        """ the number of observations made """
        return self.n

    def mean(self):
        """ the simple average of the monitored variable"""
        try: return self.total/self.n
        except: print "SimPy: Zero observations for mean"

    def var(self):
        """ the sample variance of the monitored variable """
        tot = self.total
        try: return (self.ttotal - tot*tot/self.n)/self.n
        except: print "SimPy: Zero observations for variance"
        
    def timeAverage(self,t=None):
        """ the time-average of the monitored variable.

            If t is used it is assumed to be the current time,
            otherwise t =  now()
        """
        try:
            if t == None: t = now()
            return self.Stx/(t-self.startTime)
        except:
            print "SimPy: Zero time elapsed in timeAverage"
                
    def __str__(self):
        rslt = "Monitor %s n=%d total= %7.4f"%(self.name,self.n,self.total)
        if self.Stx != 0.0: rslt= rslt+" Stx =%7.4f"%(self.Stx,)
        return rslt

